<?php

namespace ZPOS\API;

use WP_REST_Server;
use WC_REST_Customers_Controller;

class Customers extends WC_REST_Customers_Controller
{
	protected $namespace = 'wc-pos';

	public function register_routes()
	{
		parent::register_routes();

		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base . '/ids',
			[
				'methods' => WP_REST_Server::READABLE,
				'callback' => [$this, 'get_all_ids'],
				'args' => $this->get_collection_params_ids()
			]
		);

		register_rest_route($this->namespace, '/' . $this->rest_base . '/(?P<email>[\D]+)', [
			'args' => [
				'email' => [
					'description' => __('Unique email identifier for the resource.', 'woocommerce'),
					'type' => 'string',
				],
			],
			[
				'methods' => WP_REST_Server::READABLE,
				'callback' => [$this, 'get_single_item'],
				'permission_callback' => [$this, 'get_single_item_permissions_check'],
				'args' => [
					'context' => $this->get_context_param(['default' => 'view']),
				],
			]
		]);
	}

	public function get_collection_params_ids()
	{
		$params = [];

		$params['role'] = [
			'description' => __('Limit result set to resources with a specific role.', 'woocommerce'),
			'type' => 'string',
			'default' => 'customer',
			'enum' => array_merge(['all'], $this->get_role_names()),
			'validate_callback' => 'rest_validate_request_arg',
		];
		return $params;
	}

	public function get_all_ids($request)
	{
		$prepared_args = [];
		if ('all' !== $request['role']) {
			$prepared_args['role'] = $request['role'];
		}

		if (isset($request['updated_at_min']) && !empty($request['updated_at_min'])) {
			$prepared_args['meta_key']      = '_user_modified_gmt';
			$prepared_args['meta_value']    = date('Y-m-d H:i:s', strtotime($request['updated_at_min']));
			$prepared_args['meta_compare']  = '>';
		}

		$prepared_args = apply_filters('woocommerce_rest_customer_query', $prepared_args, $request);

		$query = new \WP_User_Query($prepared_args);

		return array_map(function ($user) {
			return $user->ID;
		}, $query->results);
	}

	public function get_single_item($request)
	{
		$email = $request['email'];
		$user_data = get_user_by('email', $email);

		if (empty($email) || empty($user_data->ID)) {
			return new \WP_Error('woocommerce_rest_invalid_email', __('Customer not find.', 'woocommerce'), ['status' => 404]);
		}

		$customer = $this->prepare_item_for_response($user_data, $request);
		$response = rest_ensure_response($customer);

		return $response;
	}

	public function add_additional_fields_to_object($data, $request)
	{
		$data = parent::add_additional_fields_to_object($data, $request);
		try {
			$customer = new \WC_Customer($data['id']);
			$data['taxes'] = array_values(Taxes::get_current_taxes_rates($customer, ''));
		} catch (\Exception $exception) {
			$data['taxes'] = [];
		}
		return $data;
	}

	public function get_single_item_permissions_check()
	{
		return current_user_can('read_woocommerce_pos_single_customers');
	}
}
