<?php

namespace ZPOS;

abstract class FrontendBase
{
	public $slug;
	public $regex;

	protected $full_mode_roles = ['administrator', 'customer', 'shop_manager', 'cashier'];
	protected $kiosk_mode_roles = ['kiosk'];

	public function __construct()
	{
		$this->slug = PLUGIN_NAME;
		$this->regex = '^' . $this->slug . '/?.*$';

		add_action('init', [$this, 'init']);
	}

	public function init()
	{
		add_rewrite_tag('%' . $this->slug . '%', '([^&]+)');
		add_rewrite_rule($this->regex, add_query_arg($this->slug, '1', 'index.php'), 'top');
		add_filter('option_rewrite_rules', [$this, 'rewrite_rules'], 1);
		add_action('template_redirect', [$this, 'template_redirect'], 1);
		add_action('login_enqueue_scripts', [$this, 'login_scripts']);
	}

	public function rewrite_rules($rules)
	{
		return isset($rules[$this->regex]) ? $rules : false;
	}

	public function is_pos()
	{
		global $wp;
		if (isset($wp->query_vars[$this->slug]) && $wp->query_vars[$this->slug] == 1) {
			return true;
		}

		return false;
	}

	protected function pre_template_redirect()
	{
		// check is pos
		if (!$this->is_pos()) {
			return true;
		}

		// check auth
		if (!is_user_logged_in()) {
			add_filter('login_url', [$this, 'login_url']);
			auth_redirect();
		}

		// check privileges
		if (!$this->current_user_can_access()) {
			wp_die(__('You do not have sufficient permissions to access this page.'));
		}
		return false;
	}

	public function template_redirect()
	{
		if ($this->pre_template_redirect()) {
			return;
		}

		do_action('zpos_frontend_pre_template_redirect');

		$this->post_template_redirect();

		exit();
	}

	protected function post_template_redirect()
	{
		// disable cache plugins
		$this->no_cache();

		$this->render();
	}

	protected function current_user_can_access()
	{
		$roles = wp_get_current_user()->roles;
		$allow_modes = array_merge($this->full_mode_roles, $this->kiosk_mode_roles);
		$can = count(array_diff($allow_modes, $roles)) < count($allow_modes);
		return current_user_can('access_woocommerce_pos') && $can;
	}

	public function get_user_mode()
	{
		$roles = wp_get_current_user()->roles;
		if (count(array_diff($this->full_mode_roles, $roles)) < count($this->full_mode_roles)) {
			return +1;
		}

		if (count(array_diff($this->kiosk_mode_roles, $roles)) < count($this->kiosk_mode_roles)) {
			return 0;
		}

		return -1;
	}

	public function login_url($login_url)
	{
		return add_query_arg('redirect_to', home_url($this->slug), $login_url);
	}

	private function no_cache()
	{
		// disable W3 Total Cache minify
		if (!defined('DONOTMINIFY')) {
			define("DONOTMINIFY", "true");
		}

		// disable WP Super Cache
		if (!defined('DONOTCACHEPAGE')) {
			define("DONOTCACHEPAGE", "true");
		}
	}

	public function render()
	{
		if (isset($_GET['force']) && $_GET['force'] === 'logout') {
			wp_logout();
			wp_redirect(home_url($this->slug));
			exit();
		}
		$plugin_data = get_plugin_data(PLUGIN_ROOT_FILE);
		$user = wp_get_current_user();

		$hide_errors =
			!(defined('WP_DEBUG') && defined('WP_DEBUG_DISPLAY') && WP_DEBUG && WP_DEBUG_DISPLAY) ||
			0 === intval(ini_get('display_errors'));

		$menu = [
			[
				'title' => 'Products',
				'url' => admin_url('edit.php?post_type=product'),
				'icon' => 'th-large',
			],
			[
				'title' => 'Orders',
				'url' => admin_url('edit.php?post_type=shop_order'),
				'icon' => 'list-alt',
			],
			defined('\Zhours\ACTIVE') && \Zhours\ACTIVE
				? [
					'title' => 'Hours',
					'url' => \Zhours\Aspect\Page::get('order hours')->getUrl(),
					'icon' => 'clock',
				]
				: null,
			[
				'title' => 'Reports',
				'url' => admin_url('admin.php?page=wc-reports'),
				'icon' => 'chart-area',
			],
			[
				'title' => 'Customers',
				'url' => admin_url('users.php?role=customer'),
				'icon' => 'user',
			],
			[
				'title' => 'Coupons',
				'url' => admin_url('edit.php?post_type=shop_coupon'),
				'icon' => 'tag',
			],
			[
				'title' => 'Dashboard',
				'url' => admin_url(),
				'icon' => 'tachometer',
			],
			[
				'title' => 'Settings',
				'url' => admin_url('admin.php?page=pos'),
				'icon' => 'cog',
			],
		];
		?>
		<!DOCTYPE html>
		<html>
		<head>
			<title>POS</title>
			<meta name="viewport" content="width=device-width,initial-scale=1.0,user-scalable=no">
			<script>
				window.WEBPACK_PUBLIC_PATH = "<?= Plugin::getUrl('assets/core/', true); ?>";
				window.WC_NONCE = "<?= wp_create_nonce('wp_rest'); ?>";
				window.WC_REST = "<?= esc_url_raw(rest_url('/wc-pos/')); ?>";
				window.POS_VERSION = "<?= $plugin_data['Version']; ?>";
				window.LOGOUT = "<?= str_replace('&amp;', '&', wp_logout_url(home_url($this->slug))); ?>";
				window.FORCE_LOGOUT = "<?= add_query_arg('force', 'logout', home_url($this->slug)); ?>";
				window.USERNAME = "<?= !empty($user->user_nicename) ? $user->user_nicename : $user->user_login; ?>";
				window.FULL_NAME = "<?= !empty($user->first_name) || !empty($user->last_name)
    	? $user->first_name . ' ' . $user->last_name
    	: null; ?>";
				window.MENU = <?= json_encode($menu); ?>;
				window.TABS = <?= json_encode(get_option('pos_tabs')); ?>;
				window.USER_MODE = <?= json_encode($this->get_user_mode()); ?>;
				<?php
    $image_id = get_option('pos_logo');
    $image = wp_get_attachment_image_src($image_id);
    if ($image) {
    	$src = $image[0];
    } else {
    	$src = null;
    }
    ?>
				window.LOGO = "<?= $src; ?>";
				window.IS_MOBILE_APP = <?= json_encode(Plugin::isMobileApp()); ?>;
				<?php if (!$hide_errors) { ?>
				alert("Debug mode enabled.\nDebug mode may cause runtime error and performance issues. Do not use debug mode in production.");
				<?php } ?>
			</script>
			<?php if (class_exists('\WC_Gateway_Stripe')) { ?>
				<script id="stripe-js" src="https://js.stripe.com/v3/" async></script>
			<?php } ?>
			<?php if (Plugin::getManifest('app.css')) { ?>
				<link rel="stylesheet" href="<?= Plugin::getAssetUrl('app.css'); ?>">
			<?php } ?>
		</head>
		<body>
		<div id="root"></div>
		<script src="<?= Plugin::getAssetUrl('commons.js'); ?>"></script>
		<script src="<?= Plugin::getAssetUrl('app.js'); ?>"></script>
		</body>
		</html>
		<?php
	}

	public function login_scripts()
	{
		wp_enqueue_script('login', Plugin::getAssetUrl('login.js', false), ['jquery']);
	}

	public function user_assign_message_error()
	{
		$message = [
			'Change your User Assignments to access the POS system.',
			'<a style="margin: 20px auto 0 auto; display: block; width: 250px; text-align: center;" class="button" href="' .
				add_query_arg('page', 'pos#/users', admin_url('admin.php')) .
				'">Go to User Assignment Settings</a>',
		];
		wp_die(implode("<br>", $message));
	}
}
