<?php

namespace ZPOS\Gateway;

use WC_Order;
use ZPOS\API;

class Stripe extends Base
{
	public $id = 'pos_stripe';
	public $method_title = 'POS Stripe Card';
	public $method_description = 'Stripe card payment method';
	public $has_fields = true;
	public $supports = ['products', 'pos', 'kiosk'];
	protected $stripe;
	protected $stripe_import_fields = ['publishable_key', 'testmode', 'capture', 'three_d_secure', 'secret_key'];
	protected $public_stripe_import_fields = ['publishable_key', 'testmode', 'capture', 'three_d_secure'];

	public function __construct()
	{
		parent::__construct();
		$this->stripe = new \WC_Gateway_Stripe();
		$this->init_settings();

		$this->title = $this->get_option('title');
		$this->description = $this->get_option('description');

		// Actions
		add_action('woocommerce_pos_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
	}

	public function init_settings()
	{
		parent::init_settings();

		$settings = $this->stripe->settings;

		if ($settings['testmode'] === 'yes') {
			$settings['publishable_key'] = $settings['test_publishable_key'];
			$settings['secret_key'] = $settings['test_secret_key'];
		}
		$this->settings = array_merge($this->settings, array_filter($settings, function ($key) {
			return in_array($key, $this->stripe_import_fields);
		}, ARRAY_FILTER_USE_KEY));
	}

	public function init_form_fields()
	{
		parent::init_form_fields();
		$this->stripe->init_form_fields();
		$this->form_fields = array_filter($this->stripe->form_fields, function ($key) {
			return in_array($key, $this->stripe_import_fields);
		}, ARRAY_FILTER_USE_KEY);
	}

	public function rest_filter_settings($settings)
	{
		return array_filter($settings, function ($key) {
			return in_array($key, $this->public_stripe_import_fields);
		}, ARRAY_FILTER_USE_KEY);
	}

	public function process_payment($order_id)
	{
		// get order object
		if ($order_id instanceof WC_Order) {
			$order = $order_id;
		} else {
			$order = new WC_Order($order_id);
		}

		// update pos_cash data
		$data = API::get_raw_data();
		try {
			if (isset($data['payment_details']['stripe_source']) && $data['payment_details']['stripe_source']) {
				// payment complete
				$stripe = Stripe\API::instance();
				$tendered = $order->get_meta('pos-cash-tendered', true);
				$stripe_data = $stripe->createCharge($data['payment_details']['stripe_source']['id'], $tendered, $order->get_currency(), $order->get_id());

				$order->set_transaction_id($stripe_data->transaction_id);
				$order->add_meta_data('Net Revenue From Stripe', $stripe_data->net);
				$order->add_meta_data('Stripe Fee', $stripe_data->fee);
				$order->add_meta_data('_stripe_source_id', $stripe_data->source);
				$order->add_meta_data('_stripe_charge_captured', $stripe_data->captured);

				$order->payment_complete();
				$order->set_status('completed');
				$order->save();

				// success
				return [
					'result' => 'success'
				];
			} else {
				return [
					'result' => 'failed'
				];
			}
		} catch (\Exception $e) {
			$order->add_meta_data('_pos_gateway_error', $e->getMessage());
			return [
				'result' => 'failed'
			];
		}
	}
}
